<?php

namespace Drupal\url_text\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'url' widget.
 *
 * @FieldWidget(
 *   id = "url_textfield",
 *   label = @Translation("URLfield"),
 *   field_types = {
 *     "url"
 *   }
 * )
 */
class UrlTextfieldWidget extends WidgetBase {

  private array $schemes;

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    return [
      'size' => 60,
      'placeholder' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $element['size'] = [
      '#type' => 'number',
      '#title' => $this->t('Size of Textfield'),
      '#default_value' => $this->getSetting('size'),
      '#required' => TRUE,
      '#min' => 12,
    ];
    $element['placeholder'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Placeholder'),
      '#default_value' => $this->getSetting('placeholder'),
      '#description' => $this->t('Text that will be shown inside the field until a value is entered. This hint is usually a sample value or a brief description of the expected format.'),
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    $summary = [];

    $summary[] = $this->t('Textfield size: @size', ['@size' => $this->getSetting('size')]);
    $placeholder = $this->getSetting('placeholder');
    if (!empty($placeholder)) {
      $summary[] = $this->t('Placeholder: @placeholder', ['@placeholder' => $placeholder]);
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $this->schemes = $this->getSchemes($items);

    $element += [
      '#title' => $this->t("URL"),
      '#type' => 'textfield',
      '#default_value' => $items[$delta]->value ?? NULL,
      '#size' => $this->getSetting('size'),
      '#placeholder' => $this->getSetting('placeholder'),
      '#maxlength' => $this->getFieldSetting('max_length'),
      '#attributes' => ['class' => ['url-text']],
      '#description' => "Enter a URL. Allowed schemes are: " . $this->createDescription() . ".",
      '#element_validate' => [ [$this, 'validate'] ],
    ];

    return ['value' => $element];
  }

  /**
   * Validate the scheme selection and format.
   */
  public function validate($element, FormStateInterface $form_state): void {
    $value = $element['#value'];
    if ($value) {
      // Find the end of the scheme text.
      $pos = strpos($value, '://');
      if ($pos !== FALSE) {
        // Get the full scheme
        $scheme = substr($value, 0, $pos + 3);
        // Get just the text portion of the scheme
        $scheme_type = substr($value, 0, $pos);
        // Verify that the scheme is enabled for this field.
        if (!in_array($scheme_type, $this->schemes)) {
          $form_state->setError($element, $scheme_type . $this->t(' is not an enabled URL scheme.'));
        }

        // Verify that the full text of the field matches the scheme format.
        $preg_match = $this->testRegex($value, $scheme_type);
        if (!$preg_match) {
          $form_state->setError($element, $this->t('Invalid format for ' . $scheme));
        }
      }
    }
  }

  /**
   * Generate the URL text field description based on allowed schemes.
   */
  private function createDescription(): string {

    return implode(', ',$this->schemes);
 }

  /**
   * Retrieve the allowed schemes from the field settings.
   */
  private function getSchemes(FieldItemListInterface $items) {
    $def = $items->getFieldDefinition();

    return $def->getSetting('allowed_schemes');
  }

  /**
   * Validate the field text against the chosen scheme's pattern.
   */
  private function testRegex($value, $scheme): array {
    $pattern = [
      'http' => '/^https?:\/\/(?:www\.)?[-a-zA-Z0-9@:%._\+~#=]{1,256}\.[a-zA-Z0-9()]{1,6}\b(?:[-a-zA-Z0-9()@:%_\+.~#?&\/=]*)$/',
      'https' => '/^https?:\/\/(?:www\.)?[-a-zA-Z0-9@:%._\+~#=]{1,256}\.[a-zA-Z0-9()]{1,6}\b(?:[-a-zA-Z0-9()@:%_\+.~#?&\/=]*)$/',
      'ftp' => '%^ftp:\/\/(([a-z0-9]+):([a-z0-9]+)@)*([\\.a-z0-9]+)/([\\./a-z0-9]+)$%',
      'sftp' => '%^sftp:\/\/(([a-z0-9]+):([a-z0-9]+)@)*([\\.a-z0-9]+)/([\\./a-z0-9]+)$%',
      'ssh' => '%^ssh:\/\/(([a-z0-9]+):([a-z0-9]+)@)*([\\.a-z0-9]+)/([\\./a-z0-9]+)$%',
      'mailto' => '/^mailto:\/\/[\w-\.]+@([\w-]+\.)+[\w-]{2,4}$/',
      'tel' => '^(\+\d{1,2}\s)?\(?\d{3}\)?[\s.-]\d{3}[\s.-]\d{4}$',
    ];

    preg_match($pattern[$scheme], $value, $matches);
    return $matches;
  }

}
